use nom::error::VerboseError;
use std::fmt::Debug;
use std::num::ParseFloatError;
use thiserror::Error as ThisError;

/// 
/// Тип ошибки Ingest протокола
/// 
/// К сожалению, не может быть переделан на Snafu, так как
/// Snafu не поддерживает generic типы как source ошибки,
/// не приделывая 'static лайфтайм.
/// 
/// См. https://github.com/shepmaster/snafu/issues/99 
/// 
#[allow(dead_code)]
#[derive(Debug, ThisError)]
pub enum Error<I: Debug> {
    #[error("Nom error: {0}")]
    Nom(#[from] nom::Err<VerboseError<I>>),

    #[error("Failed to parse a timestamp")]
    TimestampParseError(ParseFloatError),

    #[error("Unknown unit")]
    UnknownUnit(I),
    #[error("Failed to parse a number")]
    DecimalParsing(#[from] rust_decimal::Error),
}
