use crate::ingest_protocol::{
    parser::{parse_mac_address, parse_packet},
    SensorValue,
};
use hifitime::Epoch;

#[test]
fn test_asd() {
    let asd = r#"
#A2-C6-47-01-DF-E1#Метео
#OWNER#unknown
#T1#13.44#1000000#Outdoor
#T2#27.74#Indoor
#P1#691.02#Barometer
#LAT#55.738178
#LON#37.6068
#ALT#38
##
"#
    .trim();
    let packet = parse_packet(asd).unwrap().1;
    assert!(serde_json::to_string_pretty(&packet)
        .unwrap()
        .contains(r#""time": 1000000.0"#));
}

#[test]
fn test_mac() {
    assert_eq!(
        parse_mac_address("12-34-AA-12-55-AA").unwrap(),
        ("", [18, 52, 170, 18, 85, 170])
    );

    dbg!(Epoch::now().unwrap().to_unix_seconds());

    //println!("{:?}", parse_mac_address("12-34-AA-12-55-AA"));
}

#[test]
fn test_packet() {
    let inp = r#"#26-94-1D-75-C2-F8#Метео
#OWNER#nm17
#T1#1.12
#T2#10000#1231410321#sensorName
#T2#1.2#3400005345
##"#;

    println!("{:#?}", parse_packet(inp));
}

#[test]
fn test_hex_parsing() {
    let inp = r#"
        {"mac": "T1", "value": "123", "time": "665B514B"}
    "#
    .trim();

    let sensor_value: SensorValue = serde_json::from_str(inp).unwrap();
    assert_eq!(
        sensor_value.time.unwrap().0.to_unix_seconds(),
        1717260619.0,
        "hex text"
    );

    let inp = r#"
        {"mac": "T1", "value": "123", "time": 1717260619}
    "#
    .trim();

    let sensor_value: SensorValue = serde_json::from_str(inp).unwrap();
    assert_eq!(
        sensor_value.time.unwrap().0.to_unix_seconds(),
        1717260619.0,
        "integer test"
    );

    let inp = r#"
        {"mac": "T1", "value": "123", "time": 1717260619.0}
    "#
    .trim();

    let sensor_value: SensorValue = serde_json::from_str(inp).unwrap();
    assert_eq!(
        sensor_value.time.unwrap().0.to_unix_seconds(),
        1717260619.0,
        "float test"
    );
}
