//! Модуль веб сервера.
//!
//! Все модули отвечают только за Web сторону. Такие вещи как
//! [crate::web_server::old_device_sensor_api] отвечают только за веб версию.
//!
//! TODO: Начать работу над TCP/UDP и MQTT сервером

use fred::bytes_utils::Str;
use fred::clients::Client as RedisClient;
use fred::prelude::*;

use old_app_api::old_api_handler;

pub mod app_error;
pub mod old_app_api;
pub mod old_device_sensor_api;
pub mod utils;

#[derive(Clone)]
pub struct NMAppState {
    pub redis_client: RedisClient,
}

use crate::cli::WebServerArgs;
use crate::web_server::old_device_sensor_api::device_handler;
use crate::Cli;

use ntex::web;

pub async fn server_main(args: Cli, specific_args: WebServerArgs, client: RedisClient) {
    let asd: Str = client.ping(None).await.unwrap();

    println!("Ping result: {}", asd);

    let state = NMAppState {
        redis_client: client,
    };

    web::HttpServer::new(move || {
        web::App::new()
            .state(state.clone())
            .route("/api", web::post().to(old_api_handler))
            .route("/get", web::route().to(device_handler))
            .route("/post", web::route().to(device_handler))
    })
    .bind((specific_args.addr.as_str(), specific_args.port))
    .unwrap()
    .run()
    .await
    .unwrap(); // TODO figure out how to handle what
}
