//! Модуль обработки данных для приложений конечных пользователей, которые используют старый API.

mod handlers;
mod types;

use ntex::web::types::State;
use ntex::util::Bytes;
use ntex::web;
use nom::AsBytes;
use snafu::ResultExt;
use crate::web_server::app_error::AppError;
use crate::web_server::NMAppState;
use crate::web_server::old_app_api::handlers::{app_init, version};
use crate::web_server::old_app_api::types::{AppInitRequest, MandatoryParams};
use crate::web_server::utils::redis::is_api_key_valid;

use super::app_error;


/// Обработчик запросов от приложений.
///
/// Отвечает за разделение на функции по `cmd`.
///
/// Вызывается напрямую из ntex приложения.
pub async fn old_api_handler(
    app_state: State<NMAppState>,
    body_bytes: Bytes,
) -> Result<impl web::Responder, AppError> {
    if body_bytes.len() > 10 * 1024 {
        // 10 KiB
        return Err(AppError::RequestTooLarge);
    }

    let body_bytes = body_bytes.as_bytes();

    let mandatory_params: MandatoryParams<'_> = serde_json::from_slice(body_bytes).context(app_error::JsonSnafu {})?; // TODO: Simd-JSON

    // Ignore clippy singlematch
    if mandatory_params.cmd.as_ref() == "version" { return version((), &app_state).await }

    is_api_key_valid(&app_state.redis_client, mandatory_params.api_key.as_ref()).await?;

    match mandatory_params.cmd.as_ref() {
        "appInit" => {
            let body: AppInitRequest = serde_json::from_slice(body_bytes).context(app_error::JsonSnafu {})?;

            app_init(body, &app_state).await
        }
        _ => Err(AppError::UnknownMethod { method: mandatory_params.cmd.to_string() }),
    }

    //Ok("fuck")
}
