use serde::{Deserialize, Serialize};
use std::borrow::Cow;

// fn<'de, D>(D) -> Result<T, D::Error> where D: Deserializer<'de>

#[derive(Clone, Debug, Serialize, Deserialize)]
pub struct AppInitRequest<'a> {
    #[serde(borrow)]
    pub version: Cow<'a, str>,

    #[serde(borrow)]
    pub platform: Cow<'a, str>,

    #[serde(borrow)]
    pub model: Cow<'a, str>,

    pub width: u64,
}

#[derive(Clone, Debug, Serialize, Deserialize)]
pub struct AddLikeRequest {
    /// TODO: WTF is this? i dont remember
    pub version: u64,
}

/// Обязательные параметры у JSON app API.
///
/// При обработке входящих данных производиться два запроса [serde_json::from_str]. Один вызов пытается
/// получить [MandatoryParams], другой в зависимости от `cmd`. Это позволяет не добвалять поля из
/// этой структуры в каждом специфичном типе.
#[derive(Clone, Debug, Serialize, Deserialize)]
pub struct MandatoryParams<'a> {
    #[serde(borrow)]
    pub cmd: Cow<'a, str>,

    #[serde(borrow)]
    pub lang: Cow<'a, str>,

    /// Уникальный ID клиента.
    ///
    /// Используется на подобии как куки PHPSESSID в php.
    ///
    /// См. также: <https://www.php.net/manual/en/book.session.php>
    #[serde(borrow)]
    pub uuid: Cow<'a, str>,

    #[serde(borrow)]
    pub api_key: Cow<'a, str>,
}
